{-# Language RecordWildCards #-}
module Main where

import Csound.Base
import qualified Csound.Control.Midi.LaunchKey as L
import Csound.Patch

import Ui

baseNoteMin :: D -- (Fractional a, SigOrD a) => a
baseNoteMin = cpspch 6.00

baseNoteMaj :: D -- (Fractional a, SigOrD a) => a
baseNoteMaj = cpspch 6.03


main = dacBy setMa $ do
  Knobs{..} <- getUi
  let Mixer{..} = knobs'mixer
  res <- fmap (toMaster knobs'mixer) $ toMix knobs'mixer
      [ mul 3 $ pure $ kicks knobs'kickType
      , mul 2 $ toFxs knobs'fx =<< pads knobs'tonality mixer'drone knobs'character knobs'pulse knobs'harms
      , pure $ toCuts knobs'complexity glitch
      , pure atmo
      , pure $ toCuts knobs'complexity clocks]
  return res {- dumpWav "perf.wav" res -}

toMix :: Mixer -> [SE Sig2] -> SE Sig2
toMix Mixer{..} [kicks, pads, glitch, atmo, clocks] =
  sum
    [ mul mixer'kick kicks
    , mul mixer'pad  pads
    , mul mixer'glitch glitch
    , mul mixer'space atmo
    , mul mixer'clock clocks ]

toMaster :: Mixer -> Sig2 -> Sig2
toMaster Mixer{..} = mul mixer'master

toFxs :: Fxs -> Sig2 -> SE Sig2
toFxs Fxs{..} = pure . at (mlp (150 + 6500 * fxs'cutoff) (fxs'q * 0.85) )

toCuts :: Sig -> Sig2 -> Sig2
toCuts compl (al, ar) = cfd compl (al, ar) (bbcuts al ar (4 * 110 / 120) 8 4 1 2)

kicks ty =
  cfd ty
    (loopWav 1 "samples/kick/heartbeat-110.wav")
    (mul 2 $ loopWav 1 "samples/kick/4-floor-110.wav")


pads minMaj droneVol character rytm harms =
  cfd minMaj
    (mul 1.2 $ padMin baseNoteMin droneVol character rytm harms)
    (padMaj baseNoteMaj droneVol character rytm harms)

padMin baseNote droneVol character rytm Harms{..} =
  sum
    [ mul droneVol $ cfd (1 - character)
      (readPad "min" "base-1") -- (atNote (vibhu 65) (0.5, baseNote))
      (readPad "min" "base-1") -- (atNote dreamPad (0.5, baseNote))
    , h1
    , h2
    ]

  where
    h1 = mul harms'mild $ sum
      [ pul1 $ readPad "min" "h1-1" -- atNote dreamPad (0.4, baseNote * 2)
      , pul2 $ readPad "min" "h1-2" -- atNote dreamPad (0.3, baseNote * 3)
      ]

    h2 = mul harms'disson $ sum
      [ pul3 $ readPad "min" "h2-1" -- atNote dreamPad (0.4, baseNote * 4 * 9 / 8)
      , pul1 $ readPad "min" "h2-2" -- atNote dreamPad (0.4, baseNote * 2 * 6 / 5)
      ]

    pul xs t = mul (cfd rytm 1 (smooth 0.01 $ sqrSeq xs (110 * t / 120)))

    pul1 = pul [1, 0.5, 0.25, 0.1] 4
    pul2 = pul [1, 0.5, 0.25, 1, 0.5, 0, 1, 0] 8
    pul3 = pul [1, 0.5, 0.25, 1, 0.4, 0.1, 0.8, 0.1] 16


padMaj baseNote droneVol character rytm Harms{..} =
  sum
    [ mul droneVol $ cfd (1 - character)
      (readPad "maj" "base-1") -- (atNote whaleSongPad (0.5, baseNote))
      (readPad "maj" "base-2") -- (atNote dreamPad (0.5, baseNote))
    , h1
    , h2
    ]

  where
    h1 = mul harms'mild $ sum
      [ pul1 $ readPad "maj" "h1-1" -- atNote dreamPad (0.4, baseNote * 2)
      , pul2 $ readPad "maj" "h1-2" -- atNote dreamPad (0.3, baseNote * 3)
      ]

    h2 = mul harms'disson $ sum
      [ pul3 $ readPad "maj" "h2-1" -- atNote dreamPad (0.4, baseNote * 4 * 9 / 8)
      , pul1 $ readPad "maj" "h2-2" -- atNote dreamPad (0.4, baseNote * 2 * 5 / 4)
      , pul2 $ readPad "maj" "h2-3" -- atNote dreamPad (0.4, baseNote * 2 * 5 / 2)
      ]

    pul xs t = mul (cfd rytm 1 (smooth 0.01 $ sqrSeq xs (110 * t / 120)))

    pul1 = pul [1, 0.5, 0.25, 0.1] 4
    pul2 = pul [1, 0.5, 0.25, 1, 0.5, 0, 1, 0] 8
    pul3 = pul [1, 0.5, 0.25, 1, 0.4, 0.1, 0.8, 0.1] 16


readPad :: String -> String -> SE Sig2
readPad ty subTy = pure $ loopWav 1 $ mconcat ["samples/pad/pad-", ty, "-", subTy, ".wav"]

glitch = loopWav 1 "samples/glitch/pulse-groove.wav"

atmo = loopWav 1 "samples/env/atmo-1.wav"

clocks = loopRam (-2) (110/150) 1 "samples/ticks/tick-1.wav"

rend :: D -> String -> SE Sig2 -> IO ()
rend n name x = writeHifi secs name $ mul (linseg [0, 0.25, 1, secs - 0.5, 1, 0.25, 0])  x
  where secs = n * 60

