module Ui where

import Csound.Base
import qualified Csound.Control.Midi.LaunchKey as L
import Csound.Patch

data Knobs = Knobs
  { knobs'mixer :: Mixer
  , knobs'fx    :: Fxs
  , knobs'harms :: Harms
  , knobs'character :: Sig
  , knobs'pulse :: Sig
  , knobs'complexity :: Sig
  , knobs'tonality :: Sig
  , knobs'kickType :: Sig
  }

data Mixer = Mixer
  { mixer'master  :: Sig
  , mixer'kick    :: Sig
  , mixer'pad     :: Sig
  , mixer'glitch  :: Sig
  , mixer'space   :: Sig
  , mixer'clock   :: Sig
  , mixer'drone   :: Sig
  }

data Fxs = Fxs
  { fxs'cutoff   :: Sig
  , fxs'q        :: Sig
  , fxs'delMix   :: Sig
  , fxs'delFbk   :: Sig
  }

data Harms = Harms
  { harms'mild   :: Sig
  , harms'disson :: Sig
  }

getUi :: SE Knobs
getUi = do
  [ k1, k2,  k3,  k4,  k5,  k6,  k7,  k8, k9, k10, k11, k12, k13, k14, k15, k16 ] <- L.knobs
      [ 0, 0, 0, 0, 0, 0, 0, 0
      , 0, 0, 0, 0, 1, 0, 0, 0 ]
  let mixer = Mixer
        { mixer'master  = k1
        , mixer'kick    = k10
        , mixer'pad     = k9
        , mixer'glitch  = k11
        , mixer'space   = k12
        , mixer'clock   = k4
        , mixer'drone   = k13
        }
  let fxs = Fxs
        { fxs'cutoff   = k15
        , fxs'q        = k16
        , fxs'delMix   = k13
        , fxs'delFbk   = k14
        }
  let harms = Harms
        { harms'mild   = k5
        , harms'disson = k6
        }
  let knobs = Knobs
        { knobs'mixer  = mixer
        , knobs'fx     = fxs
        , knobs'harms  = harms
        , knobs'character = k8
        , knobs'pulse = k7
        , knobs'complexity = k3
        , knobs'tonality = k14
        , knobs'kickType = k2
        }
  return knobs

